@@ -31,6 +31,18 @@ import os
 import sys
 import typing
 
+# The following typenames are re-written for public-facing type annotations.
+TYPE_REWRITES = [
+    ('~optax._src.base.GradientTransformation', 'optax.GradientTransformation'),
+    ('~optax._src.base.Params', 'optax.Params'),
+    ('~optax._src.base.Updates', 'optax.Updates'),
+    ('~optax._src.base.OptState', 'optax.OptState'),
+    ('base.GradientTransformation', 'optax.GradientTransformation'),
+    ('base.Params', 'optax.Params'),
+    ('base.Updates', 'optax.Updates'),
+    ('base.OptState', 'optax.OptState'),
+]
+
 
 def _add_annotations_import(path):
   """Appends a future annotations import to the file at the given path."""
@@ -58,8 +70,46 @@ def _recursive_add_annotations_import():
       if file.endswith('.py'):
         _add_annotations_import(os.path.abspath(os.path.join(path, file)))
 
+
+def _monkey_patch_doc_strings():
+  """Rewrite function signatures to match the public API.
+
+  This is a bit of a dirty hack, but it helps ensure that the public-facing
+  docs have the correct type names and crosslinks.
+
+  Since all optax code lives in a `_src` directory, and since all function
+  annotations use types within that private directory, the public facing
+  annotations are given relative to private paths.
+
+  This means that the normal documentation generation process does not give
+  the correct import paths, and the paths it does give cannot cross link to
+  other parts of the documentation.
+
+  Do we really need to use the _src structure for optax?
+
+  Note, class members are not fixed by this patch, only function
+    parameters. We should find a way to genearlize this solution.
+  """
+  import sphinx_autodoc_typehints
+  original_process_docstring = sphinx_autodoc_typehints.process_docstring
+
+  def new_process_docstring(app, what, name, obj, options, lines):
+    result = original_process_docstring(app, what, name, obj, options, lines)
+
+    for i in range(len(lines)):
+      l = lines[i]
+      for before, after in TYPE_REWRITES:
+        l = l.replace(before, after)
+      lines[i] = l
+
+    return result
+
+  sphinx_autodoc_typehints.process_docstring = new_process_docstring
+
+
 if 'READTHEDOCS' in os.environ:
   _recursive_add_annotations_import()
+  _monkey_patch_doc_strings()
 
 typing.get_type_hints = lambda obj, *unused: obj.__annotations__
 sys.path.insert(0, os.path.abspath('../'))
@@ -123,7 +173,7 @@ html_theme = 'sphinx_rtd_theme'
 # Add any paths that contain custom static files (such as style sheets) here,
 # relative to this directory. They are copied after the builtin static files,
 # so a file named "default.css" will overwrite the builtin "default.css".
-html_static_path = ['_static']
+html_static_path = []
 # html_favicon = '_static/favicon.ico'
 
 # -- Options for myst -------------------------------------------------------
