@@ -42,7 +42,7 @@ def chain(
   init_fns, update_fns = zip(*args)
 
   def init_fn(params):
-    return [fn(params) for fn in init_fns]
+    return tuple(fn(params) for fn in init_fns)
 
   def update_fn(updates, state, params=None):
     if len(update_fns) != len(state):
@@ -53,7 +53,7 @@ def chain(
     for s, fn in zip(state, update_fns):
       updates, new_s = fn(updates, s, params)
       new_state.append(new_s)
-    return updates, new_state
+    return updates, tuple(new_state)
 
   return base.GradientTransformation(init_fn, update_fn)
 
