@@ -1,4 +1,3 @@
-# Lint as: python3
 # Copyright 2019 DeepMind Technologies Limited. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -25,6 +24,7 @@ import jax
 import jax.numpy as jnp
 import numpy as np
 
+from optax._src import clipping
 from optax._src import schedule
 from optax._src import transform
 from optax._src import wrappers
@@ -281,6 +281,26 @@ class ExponentialTest(chex.TestCase):
     np.testing.assert_allclose(
         expected_vals, np.array(generated_vals), atol=1e-3)
 
+  @chex.all_variants()
+  def test_immutable_count(self):
+    """Checks constant schedule for exponential decay schedule."""
+    num_steps = 5
+    # Get schedule function.
+    init_value = 32.
+    schedule_fn = self.variant(
+        schedule.exponential_decay(
+            init_value=init_value, transition_steps=1,
+            decay_rate=0.5))
+    # Test that generated values equal the expected schedule values.
+    generated_vals = []
+    for count in range(num_steps):
+      # Jax arrays are read-only in ChexVariantType.WITHOUT_DEVICE.
+      immutable_count = jnp.array(count, dtype=jnp.float32)
+      generated_vals.append(schedule_fn(immutable_count))
+    expected_vals = np.array([32, 16, 8, 4, 2], dtype=np.float32)
+    np.testing.assert_allclose(
+        expected_vals, np.array(generated_vals), atol=1e-3)
+
 
 class CosineDecayTest(chex.TestCase):
 
@@ -349,6 +369,45 @@ class CosineDecayTest(chex.TestCase):
         np.array(generated_vals), atol=1e-3)
 
 
+class WarmupCosineDecayTest(chex.TestCase):
+
+  @chex.all_variants()
+  @parameterized.named_parameters(
+      ('with end value', 10, 0.5, 1e-4),
+      ('without end value', 5, 3, 0.),)
+  def test_limits(self, init_value, peak_value, end_value):
+    """Check cosine schedule decay for the entire training schedule."""
+    schedule_fn = self.variant(schedule.warmup_cosine_decay_schedule(
+        init_value=init_value,
+        peak_value=peak_value,
+        warmup_steps=100,
+        decay_steps=1000,
+        end_value=end_value,
+    ))
+
+    np.testing.assert_allclose(init_value, schedule_fn(0))
+    np.testing.assert_allclose(peak_value, schedule_fn(100))
+    np.testing.assert_allclose(end_value, schedule_fn(1000), rtol=1e-3)
+
+
+class SGDRTest(chex.TestCase):
+
+  @chex.all_variants()
+  @parameterized.named_parameters(
+      ('with step decay', 1.6, 0.8, 0.4),
+      ('without step_decay', 1.6, 1.6, 1.6),)
+  def test_limits(self, lr0, lr1, lr2):
+    """Check cosine schedule decay for the entire training schedule."""
+    lr_kwargs = []
+    for step, lr in zip([2e3, 3e3, 5e3], [lr0, lr1, lr2]):
+      lr_kwargs += [dict(decay_steps=int(step), peak_value=lr,
+                         init_value=0, end_value=0.0, warmup_steps=500)]
+    schedule_fn = self.variant(schedule.sgdr_schedule(lr_kwargs))
+    np.testing.assert_allclose(lr0, schedule_fn(500))
+    np.testing.assert_allclose(lr1, schedule_fn(2500))
+    np.testing.assert_allclose(lr2, schedule_fn(5500))
+
+
 class PiecewiseInterpolateTest(chex.TestCase):
 
   @chex.all_variants()
@@ -496,7 +555,7 @@ class InjectHyperparamsTest(chex.TestCase):
 
   @chex.all_variants
   def test_overriding_hyperparam(self):
-    optim = schedule.inject_hyperparams(transform.clip_by_global_norm)(0.1)
+    optim = schedule.inject_hyperparams(clipping.clip_by_global_norm)(0.1)
     params = jnp.zeros((3, 5, 7))
     state = self.variant(optim.init)(params)
     update_fn = self.variant(optim.update)
