@@ -29,6 +29,8 @@ from optax._src import wrappers
 
 # pylint:disable=no-value-for-parameter
 
+_abs_sq = numerics.abs_sq
+
 
 class TraceState(NamedTuple):
   """Holds an aggregation of past updates."""
@@ -76,17 +78,39 @@ def trace(
 
 
 def _update_moment(updates, moments, decay, order):
-  """Compute the exponential moving average of the `order-th` moment."""
+  """Compute the exponential moving average of the `order`-th moment."""
   return jax.tree_multimap(
       lambda g, t: (1 - decay) * (g ** order) + decay * t, updates, moments)
 
 
+def _update_moment_per_elem_norm(updates, moments, decay, order):
+  """Compute the EMA of the `order`-th moment of the element-wise norm."""
+
+  def orderth_norm(g):
+    if jnp.isrealobj(g):
+      return g ** order
+    else:
+      half_order = order / 2
+      # JAX generates different HLO for int and float `order`
+      if half_order.is_integer():
+        half_order = int(half_order)
+      return _abs_sq(g) ** half_order
+
+  return jax.tree_multimap(
+      lambda g, t: (1 - decay) * orderth_norm(g) + decay * t, updates, moments)
+
+
 def _bias_correction(moment, decay, count):
   """Perform bias correction. This becomes a no-op as count goes to infinity."""
   bias_correction = 1 - decay**count
   return jax.tree_map(lambda t: t / bias_correction.astype(t.dtype), moment)
 
 
+def _reject_complex(params):
+  if any(jnp.iscomplexobj(x) for x in jax.tree_leaves(params)):
+    raise ValueError('This transformation does not support complex parameters.')
+
+
 class EmaState(NamedTuple):
   """Holds an exponential moving average of past updates."""
   count: chex.Array  # shape=(), dtype=jnp.int32.
@@ -124,12 +148,12 @@ def ema(
 
   def update_fn(updates, state, params=None):
     del params
-    new_ema = _update_moment(updates, state.ema, decay, order=1)
+    updates = new_ema = _update_moment(updates, state.ema, decay, order=1)
     count_inc = utils.safe_int32_increment(state.count)
     if debias:
-      new_ema = _bias_correction(new_ema, decay, count_inc)
+      updates = _bias_correction(new_ema, decay, count_inc)
     state_ema = utils.cast_tree(new_ema, accumulator_dtype)
-    return new_ema, EmaState(count=count_inc, ema=state_ema)
+    return updates, EmaState(count=count_inc, ema=state_ema)
 
   return base.GradientTransformation(init_fn, update_fn)
 
@@ -165,7 +189,7 @@ def scale_by_rss(
   def update_fn(updates, state, params=None):
     del params
     sum_of_squares = jax.tree_multimap(
-        lambda g, t: jnp.square(g) + t, updates, state.sum_of_squares)
+        lambda g, t: _abs_sq(g) + t, updates, state.sum_of_squares)
     inv_sqrt_g_square = jax.tree_map(
         lambda t: jnp.where(t > 0, jax.lax.rsqrt(t + eps), 0.0), sum_of_squares)
     updates = jax.tree_multimap(
@@ -206,7 +230,7 @@ def scale_by_rms(
 
   def update_fn(updates, state, params=None):
     del params
-    nu = _update_moment(updates, state.nu, decay, 2)
+    nu = _update_moment_per_elem_norm(updates, state.nu, decay, 2)
     updates = jax.tree_multimap(
         lambda g, n: g * jax.lax.rsqrt(n + eps), updates, nu)
     return updates, ScaleByRmsState(nu=nu)
@@ -248,10 +272,10 @@ def scale_by_stddev(
   def update_fn(updates, state, params=None):
     del params
     mu = _update_moment(updates, state.mu, decay, 1)
-    nu = _update_moment(updates, state.nu, decay, 2)
+    nu = _update_moment_per_elem_norm(updates, state.nu, decay, 2)
     updates = jax.tree_multimap(
-        lambda g, m, n: g * jax.lax.rsqrt(n - jnp.square(m) + eps), updates, mu,
-        nu)
+        lambda g, m, n: g * jax.lax.rsqrt(n - _abs_sq(m) + eps),
+        updates, mu, nu)
     return updates, ScaleByRStdDevState(mu=mu, nu=nu)
 
   return base.GradientTransformation(init_fn, update_fn)
@@ -300,7 +324,7 @@ def scale_by_adam(
   def update_fn(updates, state, params=None):
     del params
     mu = _update_moment(updates, state.mu, b1, 1)
-    nu = _update_moment(updates, state.nu, b2, 2)
+    nu = _update_moment_per_elem_norm(updates, state.nu, b2, 2)
     count_inc = numerics.safe_int32_increment(state.count)
     mu_hat = utils.cast_tree(_bias_correction(mu, b1, count_inc), mu_dtype)
     nu_hat = _bias_correction(nu, b2, count_inc)
@@ -326,7 +350,8 @@ def scale(
     An (init_fn, update_fn) tuple.
   """
 
-  def init_fn(_):
+  def init_fn(params):
+    del params
     return ScaleState()
 
   def update_fn(updates, state, params=None):
@@ -352,7 +377,8 @@ def scale_by_param_block_norm(
     An (init_fn, update_fn) tuple.
   """
 
-  def init_fn(_):
+  def init_fn(params):
+    del params
     return base.EmptyState()
 
   def update_fn(updates, state, params):
@@ -381,7 +407,8 @@ def scale_by_param_block_rms(
     An (init_fn, update_fn) tuple.
   """
 
-  def init_fn(_):
+  def init_fn(params):
+    del params
     return base.EmptyState()
 
   def update_fn(updates, state, params):
@@ -434,7 +461,7 @@ def scale_by_belief(
     del params
     mu = _update_moment(updates, state.mu, b1, 1)
     prediction_error = jax.tree_multimap(lambda g, m: g-m, updates, state.mu)
-    nu = _update_moment(prediction_error, state.nu, b2, 2)
+    nu = _update_moment_per_elem_norm(prediction_error, state.nu, b2, 2)
     nu = jax.tree_map(lambda v: v + eps_root, nu)
     count_inc = numerics.safe_int32_increment(state.count)
     mu_hat = _bias_correction(mu, b1, count_inc)
@@ -455,6 +482,9 @@ def scale_by_yogi(
 ) -> base.GradientTransformation:
   """Rescale updates according to the Yogi algorithm.
 
+  Supports complex numbers, see
+  https://gist.github.com/wdphy16/118aef6fb5f82c49790d7678cf87da29
+
   References:
     [Zaheer et al, 2018](https://papers.nips.cc/paper/2018/hash/90365351ccc7437a1309dc64e4db32a3-Abstract.html) #pylint:disable=line-too-long
 
@@ -480,9 +510,9 @@ def scale_by_yogi(
   def update_fn(updates, state, params=None):
     del params
     mu = _update_moment(updates, state.mu, b1, 1)
-    signed_sq = jax.tree_multimap(
-        lambda g, v: jnp.sign(v - g**2)*g**2, updates, state.nu)
-    nu = _update_moment(signed_sq, state.nu, b2, 2)
+    nu = jax.tree_multimap(
+        lambda g, v: v - (1 - b2) * jnp.sign(v - _abs_sq(g)) * _abs_sq(g),
+        updates, state.nu)
     count_inc = numerics.safe_int32_increment(state.count)
     mu_hat = _bias_correction(mu, b1, count_inc)
     nu_hat = _bias_correction(nu, b2, count_inc)
@@ -535,7 +565,7 @@ def scale_by_radam(
   def update_fn(updates, state, params=None):
     del params
     mu = _update_moment(updates, state.mu, b1, 1)
-    nu = _update_moment(updates, state.nu, b2, 2)
+    nu = _update_moment_per_elem_norm(updates, state.nu, b2, 2)
     count_inc = numerics.safe_int32_increment(state.count)
     b2t = b2**count_inc
     ro = ro_inf - 2 * count_inc * b2t / (1 - b2t)
@@ -569,7 +599,8 @@ def add_decayed_weights(
     An (init_fn, update_fn) tuple.
   """
 
-  def init_fn(_):
+  def init_fn(params):
+    del params
     return AddDecayedWeightsState()
 
   def update_fn(updates, state, params):
@@ -605,7 +636,8 @@ def scale_by_schedule(
     An (init_fn, update_fn) tuple.
   """
 
-  def init_fn(_):
+  def init_fn(params):
+    del params
     return ScaleByScheduleState(count=jnp.zeros([], jnp.int32))
 
   def update_fn(updates, state, params=None):
@@ -647,7 +679,8 @@ def scale_by_trust_ratio(
     An (init_fn, update_fn) tuple.
   """
 
-  def init_fn(_):
+  def init_fn(params):
+    del params
     return ScaleByTrustRatioState()
 
   def update_fn(updates, state, params):
@@ -700,7 +733,8 @@ def add_noise(
     An (init_fn, update_fn) tuple.
   """
 
-  def init_fn(_):
+  def init_fn(params):
+    del params
     return AddNoiseState(
         count=jnp.zeros([], jnp.int32), rng_key=jax.random.PRNGKey(seed))
 
@@ -710,12 +744,13 @@ def add_noise(
     treedef = jax.tree_structure(updates)
     count_inc = numerics.safe_int32_increment(state.count)
     variance = eta / count_inc**gamma
+    standard_deviation = jnp.sqrt(variance)
     all_keys = jax.random.split(state.rng_key, num=num_vars + 1)
     noise = jax.tree_multimap(
         lambda g, k: jax.random.normal(k, shape=g.shape, dtype=g.dtype),
         updates, jax.tree_unflatten(treedef, all_keys[1:]))
     updates = jax.tree_multimap(
-        lambda g, n: g + variance.astype(g.dtype) * n,
+        lambda g, n: g + standard_deviation.astype(g.dtype) * n,
         updates, noise)
     return updates, AddNoiseState(count=count_inc, rng_key=all_keys[0])
 
@@ -784,7 +819,8 @@ def centralize() -> base.GradientTransformation:
     An (init_fn, update_fn) tuple.
   """
 
-  def init_fn(_):
+  def init_fn(params):
+    del params
     return CentralState()
 
   def update_fn(updates, state, params=None):
@@ -824,6 +860,7 @@ def scale_by_sm3(
     return [jnp.zeros([s]) for s in p.shape]
 
   def init_fn(params):
+    _reject_complex(params)
     mu = jax.tree_map(zeros_for_dim, params)
     nu = jax.tree_map(jnp.zeros_like, params)
     return ScaleBySM3State(mu, nu)
