@@ -130,7 +130,12 @@ class TransformTest(parameterized.TestCase):
     mean, state = transform_fn(values[1], state)
     np.testing.assert_allclose(
         mean,
-        ((1 - d) * values[1] + d * values[0]) / (1 - d**2),
+        ((1 - d) * values[1] + d * (1 - d) * values[0]) / (1 - d**2),
+        atol=1e-2)
+    # The state must not be debiased.
+    np.testing.assert_allclose(
+        state.ema,
+        (1 - d) * values[1] + d * (1 - d) * values[0],
         atol=1e-2)
 
   @chex.all_variants()
@@ -202,6 +207,33 @@ class TransformTest(parameterized.TestCase):
     centralized_inputs, _ = centralizer.update(inputs, None)
     chex.assert_tree_all_close(centralized_inputs, outputs)
 
+  @chex.all_variants
+  def test_add_noise_has_correct_variance_scaling(self):
+    # Prepare to compare noise with a rescaled unit-variance substitute.
+    eta = 0.3
+    gamma = 0.55
+    seed = 314
+    noise = transform.add_noise(eta, gamma, seed)
+    noise_unit = transform.add_noise(1.0, 0.0, seed)
+
+    params = self.init_params
+    state = noise.init(params)
+    state_unit = noise_unit.init(params)
+
+    # Check the noise itself by adding it to zeros.
+    updates = jax.tree_map(jnp.zeros_like, params)
+
+    for i in range(1, STEPS + 1):
+      updates_i, state = self.variant(noise.update)(updates, state)
+      updates_i_unit, state_unit = noise_unit.update(updates, state_unit)
+
+      scale = jnp.sqrt(eta / i**gamma)
+
+      updates_i_rescaled = jax.tree_map(
+          lambda g, s=scale: g * s, updates_i_unit)
+
+      chex.assert_tree_all_close(updates_i, updates_i_rescaled, rtol=1e-4)
+
 
 if __name__ == '__main__':
   absltest.main()
